#include "stdafx.h"

/**
* We define our custom interrupt routine
* This function will be called each time we receive an interrupt from the board
* @param deviceHandle An handle to the device. We can use it to call other function in the interrupt routine
* @param fifoOverflow Specify if an overflow occurs. If yes, that means that we are not enough fast to handle the interrupts
* @param args An array of data
* @param argsCount The number of element (type WORD)
**/
void customInterruptRoutine (HANDLE	/*deviceHandle*/,
							 BYTE	fifoOverflow,
							 DWORD	interrutSource,
							 DWORD	/*interruptMask*/,
							 PWORD	/*args*/,
							 DWORD	/*argsCount*/ )
{
	static int interruptCounter = 0;

	/* check the source. Only the counter source interest us */
	if ( (interrutSource & PCI3116_COUNTER_TIMER_WATCHDOG_INTERRUPT) == PCI3116_COUNTER_TIMER_WATCHDOG_INTERRUPT )
	{
		printf("Counter interrupt : %d. Fifo overflow status : %d \n", interruptCounter, (int)fifoOverflow );
		interruptCounter++;
	}
}

/**
* Entry point
**/
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	ULONG readValue = 0;
	INT returnValue = 0;
	HANDLE	boardHandle = NULL;
	BYTE counterIndex = 0;

	/* call the util method to open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error while opening the board : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* Counter with interrupt sample.\n");
	printf("* This sample uses a counter with direction UP, output disabled,\n");
	printf("* a detection only on rising edge, and a reload value of 100. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask  the counter index */
	ReadUlong("Enter the index of the counter that you want to use (0, 1 or 2): ",0,2,&readValue);
	counterIndex = (BYTE) readValue;


	/* register and enable the interrupt routine */
	returnValue = i_PCI3116_SetBoardIntRoutine( boardHandle, customInterruptRoutine );
	if (returnValue != 0 )
	{
		printf("Error while setting the interrupt routine : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* start the selected component as counter */
	returnValue = i_PCI3116_InitAndStartCounterTimerWatchdog( boardHandle,
		PCI3116_COUNTER_FUNCTION,
		counterIndex,
		1,		/* interrupt enabled */
		100,	/* reload value */
		0,		/* not used for counter */
		0,		/* output disabled */
		0,		/* not used for counter */
		0,		/* direction up */
		0 );	/* counter level high */

	/* endless loop */
	printf("\nThe sample is now waiting 100 rising edge on the digital input %d.\nHit a key to exit the sample.\n", counterIndex);
	for (;;)
	{
		if ( _kbhit() )
		{
			break;
		}
		Sleep(100);
		DisplayRunningStatus();
	}

	/* stop the timer */
	returnValue = i_PCI3116_StopCounterTimerWatchdog(boardHandle, (BYTE)counterIndex);
	if ( returnValue != 0 )
	{
		printf("Error while stopping the counter logic : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* reset the board interrupt routine */
	returnValue = i_PCI3116_ResetBoardIntRoutine(boardHandle);
	if ( returnValue != 0 )
	{
		printf("Error while reseting the interrupt routine : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* success */
	return 0;
}

