/**
* This source code is an example about how to use the Set1DigitalOutputFunction
* It also uses the output memory and the GetDigitalOutputStatus function
**/
#include "stdafx.h"

#define VALUE_CHANNEL_0(x)	( x & 0x1 )
#define VALUE_CHANNEL_1(x)	( (x >> 1) & 0x1 )
#define VALUE_CHANNEL_2(x)	( (x >> 2) & 0x1 )
#define VALUE_CHANNEL_3(x)	( (x >> 3) & 0x1 )

/**
* Function that enables to print the current status of the output
* @param boardHandle A handle to a device
* @return 0 on success, != 0 on error
**/
static INT PrintOutputStatus( HANDLE boardHandle )
{
	INT returnValue = 0;
	BYTE outputStatus = 0;

	returnValue = i_PCI3116_Get4DigitalOutputsStatus( boardHandle, &outputStatus );
	if ( returnValue != 0 )
	{
		printf("Error while getting 4 digital outputs status : %d \n", returnValue );
		return -1;
	}

	printf("\nCurrent output status :\n");
	printf("____________________________________________________________\n");
	printf("|Channel 0 \t |Channel 1 \t |Channel 2 \t |Channel 3|\n");
	/* print the outputs status */
	printf("|%d \t\t |%d \t\t |%d \t\t |%d        |\n",
		VALUE_CHANNEL_0(outputStatus),
		VALUE_CHANNEL_1(outputStatus),
		VALUE_CHANNEL_2(outputStatus),
		VALUE_CHANNEL_3(outputStatus));

	return 0;
}

/**
* Entry point
**/
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE	boardHandle = NULL;
	int outputMemoryUsed = 0;
	int choice = 0;
	ULONG readValue = 0;
	
	printf(" \n");

	/* call the util method to open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error while opening the board : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* Set 1 digital output sample. \n");
	printf("* Enables to change the status of one channel. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* digital output memory */
	ReadUlong("Do you want to use output memory ? (0: No, 1: Yes) ",0,1,&readValue);
	outputMemoryUsed = (int) readValue;

	/* set the digital output memory */
	if ( outputMemoryUsed == 1 )
	{
		returnValue = i_PCI3116_SetDigitalOutputMemoryOn(boardHandle);
		if ( returnValue != 0 )
		{
			printf("Error while setting digital output memory to on : %d \n", (int) returnValue);
			i_PCI3116_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
	}
	/* reset the digital output memory */
	else if ( outputMemoryUsed == 0 )
	{
		returnValue = i_PCI3116_SetDigitalOutputMemoryOff(boardHandle);
		if ( returnValue != 0 )
		{
			printf("Error while setting digital output memory to off : %d \n", (int) returnValue);
			i_PCI3116_CloseBoard(boardHandle);
			_getch();
			return -1;
		}
	}

	/* print the current output status */
	returnValue = PrintOutputStatus( boardHandle );
	if ( returnValue != 0 )
	{
		_getch();
		return -1;
	}

	/* endless loop */
	do
	{
		BYTE channelChoice = 0;
		int state = 0;

		/* get the channel to set */
		ReadUlong("Which channel do you want to set : ",0,3,&readValue);
		channelChoice = (BYTE)readValue;

		/* get the state */
		ReadUlong("Status to set for the channel (0: Off, 1: On) ",0,1,&readValue);
		state = (int)readValue;

		/* set the state to ON*/
		if ( state == 1 )
		{
			/* set the output to on */
			returnValue = i_PCI3116_Set1DigitalOutputOn( boardHandle, channelChoice );
			if ( returnValue != 0 )
			{
				printf("Error while setting digital output %d to on : %d \n",channelChoice, (int) returnValue);
				i_PCI3116_CloseBoard(boardHandle);
				_getch();
				return -1;	
			}
		}
		/* set the state to OFF*/
		else if ( state == 0 )
		{
			/* check if digital memory is set or not */
			if ( outputMemoryUsed == 0 )
			{
				printf("You cannot set a channel to false since the digital output memory is not enabled \n");
				continue;
			}

			/* set the output to off */
			returnValue = i_PCI3116_Set1DigitalOutputOff( boardHandle, channelChoice );
			if ( returnValue != 0 )
			{
				printf("Error while setting digital output %d to off : %d \n",channelChoice, (int) returnValue);
				i_PCI3116_CloseBoard(boardHandle);
				_getch();
				return -1;	
			}
		}

		/* print the current output status */
		returnValue = PrintOutputStatus( boardHandle );
		if ( returnValue != 0 )
		{
			i_PCI3116_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* continue ? */
		ReadUlong("Do you want to set another channel ? (0: Yes, 1: No) ",0,1,&readValue);
		choice = (int)readValue;
	} while (choice != 1 );

	return 0;
}

