/**
* This sample demonstrate how to use the trigger functionnality of the timer
**/

#include "stdafx.h"

/**
* Entry point
**/
int _tmain( int /*argc*/, _TCHAR* /*argv[]*/ )
{
	INT returnValue = 0;
	HANDLE	boardHandle = NULL;
	BYTE timerIndex = 0;
	ULONG readValue = 0;

	/* call the util method to open a board */
	returnValue = OpenBoard(&boardHandle);
	if (returnValue != 0)
	{
		printf("Error while opening the board : %d \n", (int) returnValue);
		_getch();
		return -1;
	}

	/* display some useful informations */
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n");
	printf("* Timer with trigger sample. \n");
	printf("* The sequence is : \n");
	printf("* This sample uses a timer with output disabled, mode 2, \n");
	printf("* time base in millisecond and a reload value of 5000 ms. \n");
	printf("* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n\n");

	/* ask  the timer index */
	ReadUlong("Enter the index of the timer that you want to use (0, 1 or 2): ",0,2,&readValue);
	timerIndex = (BYTE)readValue;

	/* start the selected component as timer */
	returnValue = i_PCI3116_InitAndStartCounterTimerWatchdog( boardHandle,
		PCI3116_TIMER_FUNCTION,
		timerIndex,
		0,		/* interrupt disabled */
		5000,	/* reload value */
		PCI3116_TIME_BASE_MILLISECONDS,		/* time base millisecond */
		0,		/* output disabled */
		PCI3116_TIMER_MODE_2,				/* mode 2*/
		0,		/* not used */
		0 );	/* not used */

	/* endless loop */
	printf("\nTimer is started. Press 't' to trigger the timer and 'ESC' to exit this sample.\n", timerIndex);
	for (;;)
	{
		BYTE status = 0;
		BYTE triggerStatus = 0;
		WORD value = 0;

		/* get the timer status */
		returnValue = i_PCI3116_ReadTimerStatus(boardHandle,
			(BYTE)timerIndex,
			&status,
			&triggerStatus,
			&value);
		if ( returnValue != 0)
		{
			printf("Error while reading timer's status : %d \n", returnValue);
			/* stop the timer and exit */
			i_PCI3116_StopCounterTimerWatchdog(boardHandle, (BYTE)timerIndex);
			i_PCI3116_CloseBoard(boardHandle);
			_getch();
			return -1;
		}

		/* display the value */
		printf("Value : %d. Trigger : %d \t\t", (int)value, (int) triggerStatus );

		/* check if a key was hit */
		if ( _kbhit() )
		{
			switch ( _getch() )
			{
				/* t or T */
				case 116:
				case 84:
					/* trigger the timer */
					returnValue = i_PCI3116_TriggerCounterTimerWatchdog( boardHandle, (BYTE)timerIndex );
					if ( returnValue != 0 )
					{
						printf("Error while triggering timer : %d \n", returnValue);
						/* stop the timer and exit */
						i_PCI3116_StopCounterTimerWatchdog(boardHandle, (BYTE)timerIndex);
						i_PCI3116_CloseBoard(boardHandle);
						_getch();
						return -1;
					}
					break;
				/* esc */
				case 27:
					/* stop the timer */
					returnValue = i_PCI3116_StopCounterTimerWatchdog(boardHandle, (BYTE)timerIndex);
					if ( returnValue != 0 )
					{
						printf("Error while stopping the timer logic : %d \n", (int) returnValue);
						i_PCI3116_CloseBoard(boardHandle);
						_getch();
						return -1;
					}
					/* success */
					i_PCI3116_CloseBoard(boardHandle);
					return 0;
			}
		}
		Sleep(100);
		DisplayRunningStatus();
	}
}

