using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3116_NET
{
    /// <summary>
    /// Contains the call to the DLL functions
    /// </summary>
    public class DllImport
    {
        /// <summary>
        /// Definition of the function that must be given to use the interruption of the board
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="fifoStatus"></param>
        /// <param name="source">Source of the interruption. 1: digital input, 2: digital output, 4: timer, 8: counter</param>
        /// <param name="mask"></param>
        /// <param name="size"></param>
        /// <param name="args"></param>
        [UnmanagedFunctionPointer(CallingConvention.Cdecl)]
        public delegate void InterruptFunctionEventHandler(IntPtr handle,
                                                            byte overflow,
                                                            uint source,
                                                            uint mask,
                                                            IntPtr args,
                                                            uint argsSize
                                                            );

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_GetNumberOfBoards(ref byte boardNumber );

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_OpenBoardViaIndex(byte index, ref IntPtr deviceHandle );

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_CloseBoard(IntPtr deviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_SetBoardIntRoutine(IntPtr h_DeviceHandle, [MarshalAs(UnmanagedType.FunctionPtr)] InterruptFunctionEventHandler interruptFunction);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ResetBoardIntRoutine(IntPtr h_DeviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_SetDigitalOutputMemoryOn(IntPtr deviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_SetDigitalOutputMemoryOff(IntPtr deviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Set1DigitalOutputOn(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Set1DigitalOutputOff(IntPtr deviceHandle, byte channel);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Set4DigitalOutputsOn(IntPtr deviceHandle, byte value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Set4DigitalOutputsOff(IntPtr deviceHandle, byte value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Get1DigitalOutputStatus(IntPtr deviceHandle, byte channel, ref byte outputStatus);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Get4DigitalOutputsStatus(IntPtr deviceHandle, ref byte outputStatus);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Read1DigitalInput(IntPtr deviceHandle, byte channel, ref byte value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Read4DigitalInputs(IntPtr deviceHandle, ref byte value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_SetAnalogInputMode(IntPtr deviceHandle, byte inputMode);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_GetAnalogInputMode(IntPtr deviceHandle, ref byte inputMode);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_InitAndStartAnalogInputSequence(IntPtr deviceHandle,
            uint sequenceCount, 
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            uint delayTime,
            byte softTriggerMode,
            byte extTriggerMode,
            uint sequenceTriggerCount);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_StopAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern int l_PCI3116_ReadSequenceData(IntPtr deviceHandle, uint size, ushort[] data);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadMoreAnalogInputs(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ushort[] value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Read1AnalogInput(IntPtr deviceHandle,
            byte channelIndex,
            byte gain,
            byte polarity,
            uint convertingTime,
            byte interruptFlag,
            byte softTriggerMode,
            byte extTriggerMode,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_InitAndStartAutoRefreshAcquisition(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            uint convertingTime,
            ref IntPtr value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_TriggerAnalogAcquisition(IntPtr deviceHandle);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_GetAnalogInputChannelsNbr(IntPtr deviceHandle, ref byte analogInputCount);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_GetAnalogInputResolution(IntPtr deviceHandle, ref byte resolution);
		
		
        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_Write1AnalogOutput(IntPtr deviceHandle,
            byte channelIndex,
            byte polarity,
            ushort value );

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_WriteMoreAnalogOutputs(IntPtr deviceHandle,
            byte channelCount,
            byte[] channelArray,
            byte[] polarityArray,
            ushort[] values);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_InitAndStartCounterTimerWatchdog(IntPtr deviceHandle,
            byte function,
            byte index,
            byte interruptFlag,
            ushort reloadValue,
            byte timerWatchdogTimeBase,
            byte counterTimerOutputMode,
            byte timerMode,
            byte counterDirection,
            byte counterLevel);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_StopCounterTimerWatchdog(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_TriggerCounterTimerWatchdog(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadCounterStatus(IntPtr deviceHandle,
            byte index,
            ref byte status,
            ref byte clear,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadCounterValue(IntPtr deviceHandle,
            byte index,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ClearCounter(IntPtr deviceHandle,
            byte index);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadTimerStatus(IntPtr deviceHandle,
            byte index,
            ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadTimerValue(IntPtr deviceHandle,
            byte index,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadWatchdogStatus(IntPtr deviceHandle,
            byte index,
            ref byte status,
            ref byte softwareTriggerStatus,
            ref ushort value);

        [DllImport("PCI3116.dll")]
        public static extern short i_PCI3116_ReadWatchdogValue(IntPtr deviceHandle,
            byte index,
            ref ushort value);
    }
}
